function expe_display_papers
%EXPE_DISPLAY_PAPERS Display the synthesized piece of papers used in the
%experiments.
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

close all;

% General experiment parameters
EXPE_SURF_MIN = 100;
EXPE_SURF_MAX = 200;

% Noise amplitude (expressed in pixels in the input image)
noise_amp = 1;
% Noise distribution
noise_type = 'gaussian';

% Number of point correspondences
n_corresp = 150;


for i_expe_surf = EXPE_SURF_MIN:EXPE_SURF_MAX
    rand('twister', i_expe_surf); %#ok<RAND>
    randn('state', i_expe_surf); %#ok<RAND>

    % Load a synthetic piece of paper
    % (the .mat file contains the following variables: grid_2d_tpl,
    % grid_2d_img, grid_3d, cam_mat, and sz_grid)
    path = sprintf('data_paper_100/paper%04d.mat', i_expe_surf);
    load(path);
        
    % Extract some point corespondences
    [corresp_2d_tpl corresp_2d_img corresp_3d] = ...
        ris_extract_corresp(n_corresp, grid_2d_tpl, grid_2d_img, grid_3d); %#ok<NODEF>

    % Add noise
    [corresp_2d_tpl corresp_2d_img] = ...
        ris_add_noise(corresp_2d_tpl, corresp_2d_img, noise_type, noise_amp);
    
    
    grid_3d_x = reshape(grid_3d(1,:),sz_grid,sz_grid);
    grid_3d_y = reshape(grid_3d(2,:),sz_grid,sz_grid);
    grid_3d_z = reshape(grid_3d(3,:),sz_grid,sz_grid);
    ind = [1:5:100 100];
    
    % Display the 3D mesh
    figure(1);
    mesh(grid_3d_x(ind,ind), ...
         grid_3d_y(ind,ind), ...
         grid_3d_z(ind,ind), ...
         'FaceColor', 'none', 'EdgeColor', [0 0 0], 'linewidth', 1);
    set(gca, 'xlim', [min(vect(grid_3d_x(ind,ind))), max(vect(grid_3d_x(ind,ind)))], ...
        'ylim', [min(vect(grid_3d_y(ind,ind))), max(vect(grid_3d_y(ind,ind)))], ...
        'zlim', [min(vect(grid_3d_z(ind,ind))), max(vect(grid_3d_z(ind,ind)))]);
    hold on;
    plot3(corresp_3d(1,:), corresp_3d(2,:), corresp_3d(3,:), '.', 'MarkerSize', 16);
    hold off;
    daspect([1 1 1]);
    axis off;
    
    % Correspondences in the template
    figure(2);
    grid_2d_tpl_x = reshape(grid_2d_tpl(1,:), sz_grid, sz_grid);
    grid_2d_tpl_y = reshape(grid_2d_tpl(2,:), sz_grid, sz_grid);
    mesh(grid_2d_tpl_x(ind,ind), grid_2d_tpl_y(ind,ind), zeros(numel(ind)), ...
         'FaceColor', 'none', 'EdgeColor', 0.5*[1 1 1], 'linewidth', 1);
    hold on;
    plot3(corresp_2d_tpl(1,:), corresp_2d_tpl(2,:), 0.05*ones(size(corresp_2d_tpl(1,:))), '.', 'MarkerSize', 16);
    daspect([1 1 1]);
    view(0,90);
    axis off
    hold off;
    
%     set(gca, 'xlim', [-100 100], 'ylim', [-100 100]);
%     hold on;
%     rectangle('Position',[-100,-100,200,200]);
%     axis off
    
    % Correspondences in the input
    figure(3);
    grid_2d_img_x = reshape(grid_2d_img(1,:), sz_grid, sz_grid);
    grid_2d_img_y = reshape(grid_2d_img(2,:), sz_grid, sz_grid);
    mesh(grid_2d_img_x(ind,ind), grid_2d_img_y(ind,ind), zeros(numel(ind)), ...
         'FaceColor', 'none', 'EdgeColor', 0.5*[1 1 1], 'linewidth', 1);
    hold on;
    plot3(corresp_2d_img(1,:), corresp_2d_img(2,:), 0.05*ones(size(corresp_2d_img(1,:))), '.', 'MarkerSize', 16);
    daspect([1 1 1]);
    view(0,90);
    axis off
    hold off
    
    pause
end