function expe_real_stereo
%EXPE_REAL_STEREO Stereo reconstruction for the data of the experiment of
%"expe_real".
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

opengl neverselect
close all
%clear all

pathOld = path;
base_path = 'D:/PHD/PUBLICATIONS/ACCV2010/socp3d/packagesMP/inextensible';
path([base_path '/misc'],path)
path([base_path '/other'],path)
path([base_path '/../paper/other'],path)

%% data loading and preprocessing

% load the images
imTemplate = imread('example_perriollat/pattern3.ppm'); % template -> right
imLeft = imread('example_perriollat/left_smooth01.jpg'); % left -> right
imRight = imread('example_perriollat/right_smooth01.jpg');
az = 132;
ae = 32;

% load the cameras
load example_perriollat/cameraLeft
cameraLeft = cam;
load example_perriollat/cameraRight
cameraRight = cam;


% load the point correspondances and arrange them to get a coherent system
% of points between the three views
load('example_perriollat/smoothStereo.mat');
leftStereoPoints = input_points;
rightStereoPoints = base_points;

load('example_perriollat/smoothTemplate.mat');
templateTemplatePoints = input_points;
rightTemplatePoints = base_points;

nStereo = size(leftStereoPoints,1);
nTemplate = size(rightTemplatePoints,1);

% order the points
p1 = repmat(reshape(rightStereoPoints,[nStereo,1,2]),1,nTemplate);
p2 = repmat(reshape(rightTemplatePoints,[1,nTemplate,2]),nStereo,1);

[nth,ind] = min(sum((p1 - p2).^2,3));
leftStereoPoints = leftStereoPoints(ind,:);
rightStereoPoints = rightStereoPoints(ind,:);


%% stereo reconstruction

% triangulate the points
QStereo = zeros(4,nStereo);
for i=1:nStereo
    QStereo(:,i) = SfM_TriPoint({cameraLeft.mat,cameraRight.mat},[leftStereoPoints(i,:)' rightStereoPoints(i,:)']);
end
QStereo = QStereo(1:3,:)./repmat(QStereo(4,:),3,1);

save('QStereo.mat', 'QStereo');





% restore the path
path(pathOld)
