function [x y z] = ris_bary_to_cart(tri, tri_x, tri_y, tri_z, ind_tri, b1, b2, b3)
%RIS_BARY_TO_CART Converts barycentric coordinates to cartesian ones.
%
% SYNTAX
%  [x y z] = ris_bary_to_cart(tri, tri_x, tri_y, tri_z, ind_tri, b1, b2, b3)
%
% DESCRIPTION
%  Converts barycentric coordinates to cartesian ones.
%
% INPUT ARGUMENTS
%
%  - tri, tri_x, tri_y, tri_z: triangular mesh
%     * tri [nx3 matrix]: tri(i,:) are the 3 indices of the vertices of the
%         i-th facet.
%     * tri_x, tri_y, tri_z [px1 matrices]: cartesian coordinates of the 
%         vertices.
%  
%  - ind_tri [matrix]: ind_tri(i) indicates the facet (i.e. the line in 
%      "tri") to which the i-th point belongs.
%
%  - b1, b2, b3 [matrices, same size as ind_tri]: barycentric coordinates
%      of the points to convert.
%
% OUTPUT ARGUMENTS
%
%  - x, y, z [matrices, same size as b1]: (x(i),y(i),z(i)) are the 
%      cartesian coordinates of the point (b1(i),b2(i),b3(i)).
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

x = zeros(size(b1));
y = zeros(size(b1));
z = zeros(size(b1));

for i = 1:numel(b1)
    ind = tri(ind_tri(i),:);
    tmp = [tri_x(ind)' tri_y(ind)' tri_z(ind)'] * [b1(i) ; b2(i) ; b3(i)];
    x(i) = tmp(1);
    y(i) = tmp(2);
    z(i) = tmp(3);
end
