function [corresp_2d_tpl corresp_2d_img, corresp_3d] = ris_extract_corresp(n_corresp, grid_2d_tpl, grid_2d_img, grid_3d)
%RIS_EXTRACT_CORRESP Extract random point correspondences from the data
%obtained with "ris_rand_paper".
%
% SYNTAX
%  [corresp_2d_tpl corresp_2d_img] = ...
%      ris_extract_corresp(n_corresp, grid_2d_tpl, grid_2d_img)
%
%  [corresp_2d_tpl corresp_2d_img, corresp_3d] = ...
%      ris_extract_corresp(n_corresp, grid_2d_tpl, grid_2d_img, grid_3d)
%
% DESCRIPTION
%  Generates a random piece of paper.
%  There is no much choice for parameterizing the results. Default values
%  are given to generate data for the experiments on the RIS toolbox.
%  Heavily relies on the "paper" toolbox of M. Perriollat.
%
% INPUT ARGUMENTS
%
% OUTPUT ARGUMENTS
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

% Number of points in the input grid
n_points_grid = size(grid_2d_tpl, 2);

ind = randperm(n_points_grid);
ind = ind(1:n_corresp);

corresp_2d_tpl = grid_2d_tpl(:,ind);
corresp_2d_img = grid_2d_img(:,ind);
if nargout >= 3 && nargin >= 4
    corresp_3d = grid_3d(:, ind);
else
    perror('ris_extract_corresp:arguments', 'Incorrect number of arguments');
end