function [depth, out1, out2, out3, out4] = ris_split_parameters(parameters, n_corresp)
%RIS_SPLIT_PARAMETERS Split the parameters given (and optimized) by
%lsqnonlin (with "ris_cost") into the depths and the control points.
%
% [depth, ctrlpts] = ris_split_parameters(parameters, n_corresp)
% [depth, ctrl_x, ctrl_y, ctrl_z] = ris_split_parameters(parameters, n_corresp)
% [depth, ctrlpts, ctrl_x, ctrl_y, ctrl_z] = ris_split_parameters(parameters, n_corresp)
%
% Input arguments
%  
%  - parameters: column vector handled by lsqnonlin and mir_cost
%
%  - n_corresp: number of point correspondences.
%
% Output arguments
%
%  - depth [1 x n_corresp]: the depth of the point correspondences.
%
%  - ctrlpts [3 x n_ctrlpts]: matrix of the B-spline control points. The
%     number of control points (n_ctrlpts) is automatically determined from
%     the size of "parameters" and "n_corresp"
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

depth = parameters(1:n_corresp)';

n_ctrlpts_mono = (numel(parameters) - n_corresp) / 3;

ctrlpts_x = parameters(n_corresp+1:n_corresp+n_ctrlpts_mono)';
ctrlpts_y = parameters(n_corresp+n_ctrlpts_mono+1:n_corresp+2*n_ctrlpts_mono)';
ctrlpts_z = parameters(n_corresp+2*n_ctrlpts_mono+1:end)';

ctrlpts = [ctrlpts_x ; ctrlpts_y ; ctrlpts_z];

if nargout == 2
    out1 = ctrlpts;
elseif nargout == 4
    out1 = ctrlpts_x;
    out2 = ctrlpts_y;
    out3 = ctrlpts_z;
elseif nargout == 5
    out1 = ctrlpts;
    out2 = ctrlpts_x;
    out3 = ctrlpts_y;
    out4 = ctrlpts_z;
else
    error('ris_split_parameters:output', 'Wrong number of output arguments.');
end