function test_fd_bbs
%TEST_FD_BBS Test of the Feature-Driven B-spline (in 3D).
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

close all;

rand('twister', 3);

nptsu = 4;
nptsv = 4;

bbs = bbs_create(0, 1, nptsu, 0, 1, nptsv, 3);
ctrlpts = rand(3, nptsu*nptsv);

dnu = 30;
dnv = 30;

[du, dv] = meshgrid(linspace(bbs.umin, bbs.umax, dnu), ...
                    linspace(bbs.vmin, bbs.vmax, dnv));
du = du(:);
dv = dv(:);

tmp = bbs_eval(bbs, ctrlpts,  du, dv);
dx = reshape(tmp(1,:), dnv, dnu);
dy = reshape(tmp(2,:), dnv, dnu);
dz = reshape(tmp(3,:), dnv, dnu);

figure(1);
mesh(dx, dy, dz, 'facecolor', 'none');
daspect([1 1 1]);

[cu cv] = meshgrid(linspace(bbs.umin, bbs.umax, nptsu), ...
                   linspace(bbs.vmin, bbs.vmax, nptsv));
cu = cu(:);
cv = cv(:);
tmp = bbs_eval(bbs, ctrlpts, cu, cv);
fd_x = tmp(1,:);
fd_y = tmp(2,:);
fd_z = tmp(3,:);

C = bbs_coloc(bbs, cu, cv);
E = pinv(full(C));

W = bbs_coloc(bbs, du, dv);
tmp = W * E * [fd_x', fd_y', fd_z'];
dx2 = reshape(tmp(:,1), dnv, dnu);
dy2 = reshape(tmp(:,2), dnv, dnu);
dz2 = reshape(tmp(:,3), dnv, dnu);

figure(1);
hold on;
mesh(dx2, dy2, dz2, 'facecolor', 'none', 'edgecolor', zeros(1,3));

sqrt(sum(vec((dx-dx2).^2) + vec((dy-dy2).^2) + vec((dz-dz2).^2)))
