function test_ris
%TEST_RIS Test of our principle of reconstructio of inextensible surface
%with lsqnonlin and ris_cost.
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

tic
close all;

%% Load the data

%tmp = load('paper_creased.mat');
tmp = load('paper_sc.mat');
% corresp_2d_tpl = tmp.corresp_2d_tpl;
% corresp_3d = tmp.corresp_3d;
% corresp_2d_img = tmp.corresp_2d_img;
cam_mat_img = tmp.cam_mat_img;
mat_intr = cam_mat_img(:,1:3);
mesh_3d = tmp.mesh_3d;
mesh_2d_tpl = tmp.mesh_2d_tpl;
mesh_2d_img = tmp.mesh_2d_img;

rand('twister', 2); %#ok<RAND>
ampl_noise = 0.01;
npts = 100;
ind_corresp = randperm(size(mesh_2d_tpl,2));
ind_corresp = ind_corresp(1:npts);
corresp_2d_tpl = mesh_2d_tpl(:,ind_corresp) + ampl_noise*randn(2,npts);
corresp_3d = mesh_3d(:,ind_corresp);
corresp_2d_img = mesh_2d_img(:,ind_corresp) + ampl_noise*randn(2,npts);


%% Richard's first idea
% pts_3d_rh1 = ris_richard_errimg(corresp_2d_tpl, corresp_2d_img, ...
%     cam_mat_img, ...
%     2*sqrt(2)*ampl_noise, ... noise in the template
%     sqrt(2)*ampl_noise/2); % noise in the image


%% Richard's second idea
pts_3d_rh1 = ris_richard_errvect(corresp_2d_tpl, corresp_2d_img, ...
    cam_mat_img, ...
    0, ... noise in the template
    0.01/36); % noise in the image
%sqrt(2)*ampl_noise/2

%{

%% Create the B-spline model and the associated stuff

umin = min(min(mesh_2d_tpl(1,:)), min(corresp_2d_tpl(1,:)));
umax = max(max(mesh_2d_tpl(1,:)), max(corresp_2d_tpl(1,:)));
vmin = min(min(mesh_2d_tpl(2,:)), min(corresp_2d_tpl(2,:)));
vmax = max(max(mesh_2d_tpl(2,:)), max(corresp_2d_tpl(2,:)));

nctrlpts = 12;
bbs = bbs_create(umin, umax, nctrlpts, vmin, vmax, nctrlpts, 3);

% Bending matrix
lbds = 1*ones(nctrlpts-3);
bendmat = bbs_bending(bbs, lbds);

% Square root of the bending matrix
lbds = 1*ones(nctrlpts-3);
B = bbs_bending(bbs, lbds);
[P D] = eig(full(B));
Dpos = D;
Dpos(Dpos < 0) = 0;
sq_bendmat = P * sqrt(Dpos) / P;
sq_bendmat(abs(sq_bendmat)<=1.0e-3) = 0;
sq_bendmat = sparse(sq_bendmat);


%% Inextensibility points and distances
n_inext_pts = 37;
[inext_x, inext_y] = meshgrid(linspace(umin,umax,n_inext_pts), linspace(vmin,vmax,n_inext_pts));
pts_inext_img = [inext_x(:)' ; inext_y(:)'];

% Couples of points and associated distances
dist_thres = sqrt(2*(200/(n_inext_pts-1))^2) + 1;
vec_dist = pdist(pts_inext_img');
[ind_j ind_i] = meshgrid(1:n_inext_pts^2);
ind_i = squareform(tril(ind_i, -1));
ind_j = squareform(tril(ind_j, -1));
ind = find(vec_dist <= dist_thres);
dist_inext = vec_dist(ind);
dist_inext = dist_inext - 0.00125*dist_inext;
ind_pts_inext = [ind_i(ind) ; ind_j(ind)];

% Collocation matrix for the inextensibility points
colmat = bbs_coloc(bbs, pts_inext_img(1,:)', pts_inext_img(2,:)');
n_inext = numel(dist_inext);
Omega_ij = cell(1,n_inext);
for k = 1:n_inext
    i = ind_pts_inext(1,k);
    j = ind_pts_inext(2,k);

    w_ij = colmat(i,:) - colmat(j,:);
    tmp = w_ij' * w_ij;
    Omega_ij{k} = blkdiag(tmp, tmp, tmp);
end


%% Initialization
% [depths, bounds3D] = ...
%     ris_socp(corresp_2d_tpl, corresp_2d_img, cam_mat_img, sqrt(3)*ampl_noise);
% coloc = bbs_coloc(bbs, corresp_2d_tpl(1,:), corresp_2d_tpl(2,:));
% ctrlpts_init = (coloc'*coloc + bendmat) \ (coloc' * bounds3D');
% param_init = [depths ; ctrlpts_init(:)];
% save 'param_init.mat' param_init bounds3D
load 'param_init.mat'


%% Optimization
param_opt = ris_gauss_newton( ...
    @(param)ris_cost_jacob( ...
        corresp_2d_tpl, corresp_2d_img, 1, ...
        pts_inext_img, ind_pts_inext, dist_inext, 1, ...
        mat_intr, bbs, ...
        sq_bendmat, Omega_ij, param), ...
    param_init);
toc


%% Upper bounds calculation by M. Perriollat
[~,~,bounds_perrio] = ris_inext_tpl_bounding( ...
    corresp_2d_tpl, corresp_2d_img, ...
    cam_mat_img, ...
    sqrt(3)*ampl_noise);

%}

%% Display
figure(1);

% Ground truth 3D points
plot3(corresp_3d(1,:), corresp_3d(2,:), corresp_3d(3,:), 'k.');
daspect([1 1 1]);
%set(gca, 'xlim', [-70 105], 'ylim', [-110 110], 'zlim', [860 1030]);
hold on;

% Richard's first idea
plot3(pts_3d_rh1(1,:), pts_3d_rh1(2,:), pts_3d_rh1(3,:), 'kx');

return

% Perriollat's bounds
plot3(bounds_perrio(1,:), bounds_perrio(2,:), bounds_perrio(3,:), 'm+');

% Ground truth surface
gt_x = reshape(mesh_3d(1,:), 100, 100);
gt_y = reshape(mesh_3d(2,:), 100, 100);
gt_z = reshape(mesh_3d(3,:), 100, 100);
idx = 1:3:100;
mesh(gt_x(idx,idx), gt_y(idx,idx), gt_z(idx,idx), 'facecolor', 'none', 'edgecolor', 0.5*ones(1,3));

% 3D points of the SOCP initialization
plot3(bounds3D(1,:), bounds3D(2,:), bounds3D(3,:), 'o'); 

% B-spline fitted to the SOCP initialization
[~, ctrlpts_init] = ris_split_parameters(param_init, npts);

%dnu = 34;
%dnv = 34;
dnu = 100;
dnv = 100;
[du dv] = meshgrid(linspace(umin, umax, dnu), linspace(vmin, vmax, dnv));
du = du(:);
dv = dv(:);

tmp = bbs_eval(bbs, ctrlpts_init, du, dv);
ib_x = reshape(tmp(1,:), dnv, dnu);
ib_y = reshape(tmp(2,:), dnv, dnu);
ib_z = reshape(tmp(3,:), dnv, dnu);

mesh(ib_x, ib_y, ib_z, 'facecolor', 'none', 'edgecolor', [0 0 1]);

% Refined 3D point correspondences
[depth_opt, ctrlpts_opt] = ris_split_parameters(param_opt, npts);
pts_3d = repmat(depth_opt, 3, 1) .* (inv(mat_intr) * [corresp_2d_img ; ones(1, size(corresp_2d_img,2))]); %#ok<MINV>
plot3(pts_3d(1,:), pts_3d(2,:), pts_3d(3,:), 'r*')

% Refined B-spline surface
tmp = bbs_eval(bbs, ctrlpts_opt, du, dv);
rb_x = reshape(tmp(1,:), dnv, dnu);
rb_y = reshape(tmp(2,:), dnv, dnu);
rb_z = reshape(tmp(3,:), dnv, dnu);

mesh(rb_x, rb_y, rb_z, 'facecolor', 'none', 'edgecolor', [1 0 0]);

err_ib = mean(sqrt((gt_x(:)-ib_x(:)).^2 + (gt_y(:)-ib_y(:)).^2 + (gt_z(:)-ib_z(:)).^2));
err_rb = mean(sqrt((gt_x(:)-rb_x(:)).^2 + (gt_y(:)-rb_y(:)).^2 + (gt_z(:)-rb_z(:)).^2));

fprintf('Smooth surface errors\n');
fprintf('  Init=%f   Refined=%f\n', err_ib, err_rb);

err_perr = mean(sum((corresp_3d - bounds_perrio).^2));
err_socp = mean(sum((corresp_3d - bounds3D).^2));
err_drpt = mean(sum((corresp_3d - pts_3d).^2));
rbpt = bbs_eval(bbs, ctrlpts_opt, corresp_2d_tpl(1,:), corresp_2d_tpl(2,:));
err_rbpt = mean(sum((corresp_3d - rbpt).^2));
err_rich1 = mean(sum((corresp_3d - pts_3d_rh1).^2));
fprintf('3D points errors\n');
fprintf('  Perrio=%f   SOCP=%f   Richard1=%f   DepthRefined=%f   Refined=%f\n', ...
    err_perr, err_socp, err_rich1, err_drpt, err_rbpt);
